#include <math.h>
#include "LinePattern.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include <vector>
#include <list>
#include <iostream>
using namespace std;

//I'm using SDL_Rect as the base structure for the points you're supposed to connect. x, y are the positions.

LinePattern :: LinePattern ()
{ 
  totalNumDots = 10;

  //vector <vector <int>> angleChangeDistribution;
  angleChangeDistribution.resize (4);
  angleChangeDistribution.at(0).resize (5);
  angleChangeDistribution.at(0).at(0) = 10;
  angleChangeDistribution.at(0).at(1) = 40;
  angleChangeDistribution.at(0).at(2) = 80;
  angleChangeDistribution.at(0).at(3) = 40;
  angleChangeDistribution.at(0).at(4) = 10;

  angleChangeDistribution.at(1).resize (5);
  angleChangeDistribution.at(1).at(0) = 60;
  angleChangeDistribution.at(1).at(1) = 10;
  angleChangeDistribution.at(1).at(2) = 50;
  angleChangeDistribution.at(1).at(3) = 10;
  angleChangeDistribution.at(1).at(4) = 60;

  angleChangeDistribution.at(2).resize (5);
  angleChangeDistribution.at(2).at(0) = 50;
  angleChangeDistribution.at(2).at(1) = 30;
  angleChangeDistribution.at(2).at(2) = 30;
  angleChangeDistribution.at(2).at(3) = 30;
  angleChangeDistribution.at(2).at(4) = 50;

  angleChangeDistribution.at(3).resize (5);
  angleChangeDistribution.at(3).at(0) = 70;
  angleChangeDistribution.at(3).at(1) = 15;
  angleChangeDistribution.at(3).at(2) = 15;
  angleChangeDistribution.at(3).at(3) = 15;
  angleChangeDistribution.at(3).at(4) = 70;
}

void LinePattern :: init (SDL_Rect dim, SDL_Color & rulz)
{
  //restarts the pattern, given pattern rules.
  points.resize (0);

  //randomize drawing stuff
  ang1 = rand () % 360;
  ang2 = ang1 + 180;
  if (ang2 > 360)
    ang2 -= 360;

  sin1 = sin (double(ang1));
  cos1 = cos (double(ang1));
      
  sin2 = sin (double(ang2));
  cos2 = cos (double(ang2));

  //get new rules.
  startVector = rulz.r;
  distBetweenPoints = rulz.g;
  angleBetweenPoints = rulz.b;

  firstPoint = true;
  //other functions assume you have 2 lines
  getNewLine (dim, true);
}

void LinePattern :: render (SDL_Surface * dest, SDL_Rect dim)
{
  //draw each point on the SDL_Surface dest.

  //as a circle. You can use the w parameter to mark them as 'drawn' and 'not drawn.'

  //draw the current line.
  vector <SDL_Rect> :: iterator iter;

  Uint8 r, g, b, a;
  Uint8 r2, g2, b2, a2;
  Sint32 v;
  Uint32 helper1 = points.size () - 2;

  if (points.size () > 1)
    {
      for (Sint16 i = Sint16(points.size () - 2); i >= 0; i --)
	{
	  if (i == 0)
	    {r = 192; g = 192; b = 192; a = 255;}
	  else
	    {
	      v = -90 * (i-1) / helper1 + 270;

	      HSVtoRGB (240,360,Uint16(v),r,g,b);
	      a = 255;
	    }
	  
	  filledTrigonRGBA(dest, 
			   points.at(i+1).x + dim.x, points.at(i+1).y + dim.y, 
			   points.at(i).x + dim.x + (Sint16)(sin1 * points.at(i + 1).w), points.at(i).y  + dim.y + (Sint16)(cos1 * points.at(i+1).w), 
			   points.at(i).x + dim.x + (Sint16)(sin2 * points.at(i+1).w), points.at(i).y + dim.y + (Sint16)(cos2 * points.at(i+1).w), 
			   r, g, b, a);
	  
	  if (i == 0)
	    {r2 = 192; g2 = 192; b2 = 192; a2 = 255;}
	  else
	    {
	      v = -90 * (i-1) / helper1 + 270;

	      HSVtoRGB (240,360,Uint16(v),r2,g2,b2);
	      a2 = 255;
	    }

	  aatrigonRGBA(dest, 
			   points.at(i+1).x + dim.x, points.at(i+1).y + dim.y, 
			   points.at(i).x + dim.x + (Sint16)(sin1 * points.at(i + 1).w), points.at(i).y  + dim.y + (Sint16)(cos1 * points.at(i+1).w), 
			   points.at(i).x + dim.x + (Sint16)(sin2 * points.at(i+1).w), points.at(i).y + dim.y + (Sint16)(cos2 * points.at(i+1).w), 
			   r2, g2, b2, a2);

	  if (i == 1)
	    {r = 255; g = 255; b = 255; a = 255;}
	  else
	    {
	      v = -90 * (i-2) / helper1 + 270;

	      HSVtoRGB (240,360,Uint16(v),r,g,b);
	      a = 255;
	    }

	  if (i != 0)
	    filledCircleRGBA(dest, points.at (i).x + dim.x, points.at (i).y + dim.y, points.at (i).w, r,g,b,a);
	}
    }
  else
    {
      filledCircleRGBA(dest, points.at (0).x + dim.x, points.at (0).y + dim.y, points.at (0).w, 128,128,128,255);
      aacircleRGBA(dest, points.at(0).x + dim.x, points.at(0).y + dim.y,
		   points.at(0).w, 192, 192, 192, 255);
    }
}

int LinePattern :: getLastPoint (SDL_Rect & ptA)
{
  //returns the number of points in the vector.
  //throws an error if there are 0 points in the vector.

  ptA = points.back ();
  return points.size ();
}

int LinePattern :: getContactPoint (SDL_Rect & ptA)
{
  //sets ptA to the point the user should draw to.
  //returns the size of the vector.
  //throws an error if there are 0 points in the vector.

  if (points.size () >= 2)
    ptA = points.at (1);
  else
    ptA = points.at (0);
  return points.size ();
}

int LinePattern :: get2ndContactPoint (SDL_Rect & ptA)
{
  //sets ptA to the point the user should draw to.
  //returns the size of the vector.
  //throws an error if there are less than 3 points in the vector.

  ptA = points.at (2);
  return points.size ();
}

void LinePattern :: getNewLine (SDL_Rect dim, bool newline)
{
  //creates a new point.
  SDL_Rect newPt = generateNextPoint (dim);
  vector <SDL_Rect> :: iterator iter;
  Sint32 helper1 = totalNumDots - 2;

  //do starter stuff
  if (points.size () <= 0 && newline)
    {
      newPt.w = 8;
      newPt.h = 5;
      
      points.push_back (newPt);
    }
  else if (points.size () <= 1 && !newline)
    {
      //shrink the first point
      points.at (0).w = 1;
      points.at (0).h = 1;

      //add the newest point as the next one to click
      newPt.w = 8;
      newPt.h = 5;
      points.push_back (newPt);

      //add the rest

      for (int i = 0; i < (totalNumDots - 2); i++)
	{
	  newPt = generateNextPoint (dim);

	  newPt.w = (-4 * i / helper1) + 5;
	  newPt.h = 3;

	  //add to queue
	  points.push_back (newPt);
	}
    }
  else
    {
      for (int i = 3; i < (totalNumDots - 2); i++)
	{
	  points.at (i).w = (-4 * (i - 3) / helper1) + 5;
	}

      //shrink the 2nd point
      points.at (1).w = 1;
      points.at (1).h = 1;

      //embiggen the 3rd point
      points.at (2).w = 8;
      points.at (2).h = 5;      

      //pop the front
      points.erase (points.begin (), (points.begin () + 1) );

      //add to the back
      newPt.w = 3;
      newPt.h = 3;
      points.push_back (newPt);
    }
}

SDL_Rect LinePattern :: generateNextPoint (SDL_Rect dim)
{
  SDL_Rect newPt;
  SDL_Rect lastPt;

  Sint32 dir;
  Uint16 dist;
  Uint8 anglChoice;
  Uint8 d2;
  Sint32 anglChange;

  SDL_Rect candidateA, candidateB;
  Uint32 distFromCenterA, distFromCenterB;
  bool chooseCandidateA = false;
  Sint32 dirA, dirB;
  Uint8 weightA = 0, weightB = 0;

  //start in the lower left corner and rotate clockwise. 8 is the center.
  if (firstPoint)
    {
      newPt.x = rand () % dim.w / 3;
      newPt.y = rand () % dim.h / 3;

      switch (startVector){
      case 0: newPt.y += 2 * dim.h / 3; currentAngle = 135; break;
      case 1: newPt.y += dim.h / 3; currentAngle = 90; break;
      case 2: currentAngle = 45; break;
      case 3: newPt.x += dim.w / 3; currentAngle = 0; break;
      case 4: newPt.x += 2 * dim.w / 3; currentAngle = 315; break;
      case 5: newPt.x += 2 * dim.w / 3; newPt.y += dim.h / 3; currentAngle = 270; break;
      case 6: newPt.x += 2 * dim.w / 3; newPt.y += 2 * dim.h / 3; currentAngle = 225; break;
      case 7: newPt.x += dim.w / 3; newPt.y += 2 * dim.h / 3; currentAngle = 180; break;
      case 8:
      default: newPt.x += dim.w / 3; newPt.y += dim.h / 3; currentAngle = rand () % 360; break;
      }
      firstPoint = false;
      mainAngleIndex = 2;
    }
  else
    {
      //generate new pt according to rules
      //get info on last points
      getLastPoint (lastPt);

      //vary angle according to angleBetweenPoints and drawDirection.

      anglChoice = angleBetweenPoints;

      //every 5 points or so, reverse the drawDirection.
      //randomize for extra fun!!!

      d2 = distBetweenPoints;

      anglChoice = 1;

      //angles are in increments of 15 degrees
      anglChange = (mainAngleIndex + 2) * 15;

      //vary distance according to distBetweenPoints
      switch (d2){
      case 0: dist = 55; break;
      case 1: dist = 60; break;
      case 2: dist = 65; break;
      case 3: 
      default: dist = 70; break;
      }

      anglChange += rand () % 11 - 5;

      newPt.x = lastPt.x;
      newPt.y = lastPt.y;

      dir = currentAngle;

      //choose the directiont that brings it closer to the center.
      //it should stay at (dist) distance away from the center.
      //get the 2 candidates (A & B) then choose the one closer to the center.

      dirA = currentAngle + anglChange;
      dirB = currentAngle - anglChange;

      candidateA.x = lastPt.x + (Sint16) ((double)dist * sin ((double)dirA * 3.14159 / 180.0));
      candidateA.y = lastPt.y + (Sint16) ((double)dist * cos ((double)dirA * 3.14159 / 180.0));

      candidateB.x = lastPt.x + (Sint16) ((double)dist * sin ((double)dirB * 3.14159 / 180.0));
      candidateB.y = lastPt.y + (Sint16) ((double)dist * cos ((double)dirB * 3.14159 / 180.0));

      //if it's out of bounds, reverse and clip the next point in the boundary.
      if (boundaryNextPoint (candidateA, dim, dirA))
	addWeight (weightA, weightB, 2);

      if (boundaryNextPoint (candidateB, dim, dirB))
	addWeight (weightB, weightA, 2);

      distFromCenterA = ((candidateA.x - (dim.w / 2)) * (candidateA.x - (dim.w / 2))) + ((candidateA.y - (dim.h / 2)) * (candidateA.y - (dim.h / 2)));
      distFromCenterB = ((candidateB.x - (dim.w / 2)) * (candidateB.x - (dim.w / 2))) + ((candidateB.y - (dim.h / 2)) * (candidateB.y - (dim.h / 2)));

      if (distFromCenterA < distFromCenterB)
	addWeight (weightB, weightA, 2);
      else if (distFromCenterA > distFromCenterB)
	addWeight (weightA, weightB, 2);
      else if (rand () % 2 == 0)
	addWeight (weightA, weightB, 2);
      else
	addWeight (weightB, weightA, 2);

      if (distFromCenterA < Uint32(dist * dist / 4))
	addWeight (weightA, weightB, 10);

      if (distFromCenterB < Uint32(dist * dist / 4))
	addWeight (weightB, weightA, 10);
  
      chooseCandidateA = (weightA < weightB);

      //lines you want to end this with
      if (chooseCandidateA)
	{
	  dir = dirA;
	  newPt.x = candidateA.x;
	  newPt.y = candidateA.y;
	}
      else
	{
	  dir = dirB;
	  newPt.x = candidateB.x;
	  newPt.y = candidateB.y;
	}

      //should you keep the angle?
      if (rand () % 100 > angleChangeDistribution.at (angleBetweenPoints).at(mainAngleIndex))
	{
	  if (rand () % 2 == 0)
	    {
	      if (mainAngleIndex == 0)
		mainAngleIndex = 1;
	      else
		mainAngleIndex -= 1;
	    }
	  else
	    {
	      if (mainAngleIndex == angleChangeDistribution.size () - 1)
		mainAngleIndex -= 1;
	      else
		mainAngleIndex += 1;
	    }
	}

      //clip dir
      while (dir < 0 || dir >= 360)
	{
	  if (dir < 0)
	    dir += 360;
	  if (dir >= 360)
	    dir -= 360;
	}

      //rebound and clipping: reverse the angle, too.

      //make sure they aren't the same point;
      while (lastPt.x == newPt.x && lastPt.y == newPt.y)
	{
	  //uhh, randomize?
	  newPt.x += rand () % 21 - 10;
	  newPt.y += rand () % 21 - 10;
	}
      
      boundaryNextPoint (newPt, dim, dir);

      //update current angle
      currentAngle = dir;
    }
  return newPt;
}

void  LinePattern :: addWeight (Uint8 & wA, Uint8 & wB, Uint8 amount)
{
  //angleBetweenPoints acts as an evil AI as well -- it will try to choose the worst of the two lines.
  if (rand () % 10 < angleBetweenPoints)
    wB += amount; //evil...add weight to the WRONG Line
  else
    wA += amount; //business as usual.
}

bool LinePattern :: boundaryNextPoint (SDL_Rect & newPt, SDL_Rect dim, Sint32 & dir)
{
  //checks to see if the point newPt is out of bounds (dim)
  //if so, it bounces newPt off of dim and updates newPt and dir. Then returns true.
  bool bounced = false;

  if (newPt.x < 0)
    {
      newPt.x = 0 - newPt.x;
      
      if (dir > 180 && dir < 270)
	dir = 90 + (dir - 180);
	  
      if (dir >= 270 && dir < 360)
	dir = 90 + (dir - 360);

      bounced = true;
    }
  if (newPt.x > dim.w)
    {
      newPt.x = dim.w - newPt.x + dim.w;
      
      if (dir >= 0 && dir < 90)
	dir = 270 + (dir - 0);
      
      if (dir >= 90 && dir < 180)
	dir = 270 + (dir - 180);

      bounced = true;
    }
  if (newPt.y < 0)
    {
      newPt.y = 0 - newPt.y;
      
      if (dir > 90 && dir < 180)
	dir = 0 + (dir - 90);
      
      if (dir >= 180 && dir < 270)
	dir = 0 + (dir - 270);

      bounced = true;
    }
  if (newPt.y > dim.h)
    {
      newPt.y = dim.h - newPt.y + dim.h;
      
      if (dir >= 0 && dir < 90)
	dir = 180 + (dir - 90);
      
      if (dir > 270 && dir <= 360)
	dir = 180 + (dir - 270);

      bounced = true;
    }
  
  //clipping
  if (newPt.x < 0)
    newPt.x = 0;
  if (newPt.x > dim.w)
    newPt.x = dim.w;
  if (newPt.y < 0)
    newPt.y = 0;
  if (newPt.y > dim.h)
    newPt.y = dim.h;
  
  //clip dir
  while (dir < 0 || dir >= 360)
    {
      if (dir < 0)
	dir += 360;
      if (dir >= 360)
	dir -= 360;
    }

  return bounced;
}

Uint32 LinePattern :: chooseColor (SDL_Rect pt, SDL_Surface * dest, Uint8 & r, Uint8 & g, Uint8 & b, Uint8 & a)
{
  //picks the color, based on pt's h parameter.
  Uint32 ans;

  //do math magic here; assumed to be 5 dots long
  r = 192;
  g = 192;
  b = 192;
  a = 255 * pt.h / 5;

  ans = SDL_MapRGBA(dest -> format, r, g, b, a);
  return ans;
}

LinePattern :: ~LinePattern ()
{
}

//========================================
//Color conversion functions!
//taken from http://www.cs.rit.edu/~ncs/color/t_convert.html

void RGBtoHSV(Uint8 r, Uint8 g, Uint8 b, Uint16 & h, Uint16 & s, Uint16 & v)
{
  float h2, s2, v2;
  RGBtoHSVhelper ( (float(r)/256.0), (float(g)/256.0), (float(b)/256.0), &h2, &s2, &v2);

  h = (Uint16) h2;
  s = (Uint16) (s2 * 360);
  v = (Uint16) (v2 * 360);
}

void HSVtoRGB(Uint16 h, Uint16 s, Uint16 v, Uint8 & r, Uint8 & g, Uint8 & b)
{
  float r2, g2, b2;
  HSVtoRGBhelper (&r2, &g2, &b2, float(h), (float(s)/360.0), (float(v)/360.0));

  r = (Uint8) (r2 * 256);
  g = (Uint8) (g2 * 256);
  b = (Uint8) (b2 * 256);
}


void RGBtoHSVhelper( float r, float g, float b, float *h, float *s, float *v )
{
  float min, max, delta;
  
  min = MIN( r, g, b );
  max = MAX( r, g, b );
  *v = max;				// v
  
  delta = max - min;
  
  if( max != 0 )
    *s = delta / max;		// s
  else {
    // r = g = b = 0		// s = 0, v is undefined
    *s = 0;
    *h = -1;
    return;
  }
  
  if( r == max )
    *h = ( g - b ) / delta;		// between yellow & magenta
  else if( g == max )
    *h = 2 + ( b - r ) / delta;	// between cyan & yellow
  else
    *h = 4 + ( r - g ) / delta;	// between magenta & cyan

  *h *= 60;				// degrees
  if( *h < 0 )
    *h += 360;
  
}

void HSVtoRGBhelper( float *r, float *g, float *b, float h, float s, float v )
{
  int i;
  float f, p, q, t;
  
  if( s == 0 ) {
    // achromatic (grey)
    *r = *g = *b = v;
    return;
  }
  
  h /= 60;			// sector 0 to 5
  i = int(floor( h ));
  f = h - i;			// factorial part of h
  p = v * ( 1 - s );
  q = v * ( 1 - s * f );
  t = v * ( 1 - s * ( 1 - f ) );
	
  switch( i ) {
  case 0:
    *r = v;
    *g = t;
    *b = p;
    break;
  case 1:
    *r = q;
    *g = v;
    *b = p;
    break;
  case 2:
    *r = p;
    *g = v;
    *b = t;
    break;
  case 3:
    *r = p;
    *g = q;
    *b = v;
    break;
  case 4:
    *r = t;
    *g = p;
    *b = v;
    break;
  default:		// case 5:
    *r = v;
    *g = p;
    *b = q;
    break;
  }
}

float MIN (float a, float b, float c)
{
  float ans;

  if (a < b)
    ans = a;
  else
    ans = b;

  if (c < ans)
    ans = c;

  return ans;
}

float MAX (float a, float b, float c)
{
  float ans;

  if (a > b)
    ans = a;
  else
    ans = b;

  if (c > ans)
    ans = c;

  return ans;
}
