#include "MediaResource.h"

#include "SDL.h"
#include "SDL_image.h"
//#include "SDL_mixer.h"

#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include <ostream>
#include <sstream>
using namespace std;

#include "AnimFrame.h"

MediaResource :: MediaResource (string n)
{
  //loads stuff.
  //looks for the file: media/chars/n/info (n is what you pass in.)
  //each line in info is of the form:
  //Type Media MediaSpecificStuff
  //examples:
  //Idle 0 -1 0 <-- Media 0: Main info about the animation
  //Attk 1 10 20 900 <--- Media 1: Animation frame.
  //Dead 2 scream.ogg <---- Media 2: Sounds.

  string bufString;
  string activity;
  Uint32 mediaType;
  Sint32 intReader;
  Uint32 uintReader;
  Uint8 animType = 255;

  Uint32 picIndex = 0;

  Sint32 xo;
  Sint32 yo;
  Uint32 to;

  ostringstream fileName (ostringstream::out);
  fileName << "media/chars/" << n << "/info";

  actionList.resize (4);
  actionList.at(0) = NULL;
  actionList.at(1) = NULL;
  actionList.at(2) = NULL;
  actionList.at(3) = NULL;

  action * tempAction = NULL;
  AnimFrame * tempAF = NULL;

  //load the file.
  infile.open (fileName.str().c_str());
  if (!infile.is_open ())
    {cout << "Cannot find file:" << fileName.str () << endl; 
    cout.flush (); return;}

  name.assign (n);
  picIndex = 0;

  while (!infile.eof ())
    {
      infile >> bufString;

      if (bufString.compare ("Idle") == 0 || bufString.compare ("Pain") == 0 || bufString.compare ("Dead") == 0 || bufString.compare ("Attk") == 0)
	{
	  activity = bufString;
	  infile >> mediaType;

	  //call subfuctions depending on mediaType.

	  tempAction = NULL;
	  tempAF = NULL;

	  if (bufString.compare ("Idle") == 0)
	    animType = 0;
	  
	  if (bufString.compare ("Attk") == 0)
	    animType = 1;

	  if (bufString.compare ("Pain") == 0)
	    animType = 2;

	  if (bufString.compare ("Dead") == 0)
	    animType = 3;

	    if (animType < 4)
	      {
		switch (mediaType){
		case (0):
		  //make a new action type.
		  tempAction = new action ();
		  infile >> intReader;
		  tempAction -> numLoops = intReader;
		  infile >> uintReader;
		  tempAction -> whatToDoWhenOutOfLoops = uintReader;
		  actionList.at(animType) = tempAction;
		  picIndex = 0;
		  break;
		case (1):
		  //load animations
		  //construct the name, look for it.
		  tempAF = NULL;
		  fileName.str (*new string (""));
		  fileName << "media/chars/" << n << "/";

		  switch (animType)
		    {
		    case (0): fileName << "Idle"; break;
		    case (1): fileName << "Attk"; break;
		    case (2): fileName << "Pain"; break;
		    case (3): fileName << "Dead"; break;
		    default: break;
		    }
		  
		  fileName << picIndex << ".png";

		  //the AnimFrame does the actual loading
		  tempAF = new AnimFrame (fileName.str ());
		  if (tempAF -> isValid ())
		    {
		      infile >> xo; infile >> yo; infile >> to;
		      tempAF -> setOffsetAndTime (xo, yo, to);
		      actionList.at (animType) -> anim.push_back (tempAF);
		      picIndex ++;
		    }
		  else
		    { 
		      delete tempAF;
		    }
		  break;
		case (2):
		  //import sounds
		  break;
		default: break;
		};
	      }
	}
    }
}

Sint32 MediaResource :: changePic (Uint32 timeElapsed, Uint32 & animIndex, Uint32 & frameIndex, Sint32 & numLoops)
{
  //changes the pic.
  //returns 0 if it didn't change the pic.
  //returns 1 if it incremented the pic.
  //returns 2 if it reached the end of the pics and was looping over.
  //returns 3 if it reached the end of the pics and was holding the last frame.
  //returns 4 if it reached the end and was resetting to the idle frame.

  Sint32 ans = 0;

  Uint32 timeToHoldFrame = actionList.at (animIndex) -> anim.at (frameIndex) -> getTime ();

  if (timeElapsed >= timeToHoldFrame)
    {
      //increment frame.
      frameIndex ++;
      ans = 1;

      //check if it's too far
      if (frameIndex >= actionList.at (animIndex) -> anim.size ())
	{
	  //decrease the number of loops...maybe

	  if (numLoops <= -1)
	    {
	      //infinite loops.
	      frameIndex = 0;
	      ans = 2;
	    }
	  else if (numLoops == 0)
	    {
	      //decide what to do here.
	      switch (actionList.at (animIndex) -> whatToDoWhenOutOfLoops){
	      case 0: //just cancel the animation
		animIndex = 0;
		frameIndex = 0;
		ans = 4;
		break;
	      case 1: //hold it on the last frame
		frameIndex --;
		ans = 3;
		break;
	      default:
		animIndex = 0;
		frameIndex = 0;
		ans = 2;
		break;
	      };
	    }
	  else
	    {
	      //just loop and try again
	      numLoops --;
	      frameIndex = 0;
	      ans = 2;
	    }
	}
    }
  return ans;
}

SDL_Surface * MediaResource :: drawPic (SDL_Surface * dest, SDL_Rect & center, Uint32 animIndex, Uint32 frameIndex)
{
  //gets the next frame.
  actionList.at (animIndex) -> anim.at (frameIndex) -> render (dest, center);
  return 0;
}

Sint32 MediaResource :: getLoops (Uint8 animIndex)
{
  return actionList.at (animIndex) -> numLoops;
}

MediaResource :: ~MediaResource ()
{
  infile.close ();
}
